@@ -6,6 +6,11 @@ jomo.lmer <-
       cat("Minimum number of imputations:2. For single imputation using function jomo.lmer.MCMCchain\n")
     }
     stopifnot(is.data.frame(data))
+    if (is_tibble(data)) {
+      data<-data.frame(data)
+      warning("tibbles not supported. data converted to standard data.frame. ")
+    }
+    if (isTRUE(any(sapply(df, is.character)))) stop("Character variables not allowed in data\n")
     stopifnot(any(grepl("~",deparse(formula))))
     fit.cr<-lmer(formula,data=data, na.action = na.omit)
     betaY.start<-fixef(fit.cr)
@@ -30,22 +35,34 @@ jomo.lmer <-
     for (j in 1:ncol(Ycov)) {
       if (level[1, which(colnames(level)==colnames(Ycov)[j])]==1) {
         if (is.numeric(Ycov[,j])) {
-          if (is.null(Y.con)) Y.con<-data.frame(Ycov[,j,drop=FALSE])
-          else Y.con<-data.frame(Y.con,Ycov[,j,drop=FALSE])
+          if (is.null(Y.con)) {
+            Y.con<-data.frame(Ycov[,j,drop=FALSE])
+          } else {
+            Y.con<-data.frame(Y.con,Ycov[,j,drop=FALSE])
+          }
         }
         if (is.factor(Ycov[,j])) {
-          if (is.null(Y.cat)) Y.cat<-data.frame(Ycov[,j,drop=FALSE])
-          else Y.cat<-data.frame(Y.cat,Ycov[,j,drop=FALSE])
+          if (is.null(Y.cat)) {
+            Y.cat<-data.frame(Ycov[,j,drop=FALSE])
+          } else {
+            Y.cat<-data.frame(Y.cat,Ycov[,j,drop=FALSE])
+          }
           Y.numcat<-cbind(Y.numcat,nlevels(Ycov[,j]))
         }
       } else {
         if (is.numeric(Ycov[,j])) {
-          if (is.null(Y2.con)) Y2.con<-data.frame(Ycov[,j,drop=FALSE])
-          else Y2.con<-data.frame(Y2.con,Ycov[,j,drop=FALSE])
+          if (is.null(Y2.con)) {
+            Y2.con<-data.frame(Ycov[,j,drop=FALSE])
+          } else {
+            Y2.con<-data.frame(Y2.con,Ycov[,j,drop=FALSE])
+          }
         }
         if (is.factor(Ycov[,j])) {
-          if (is.null(Y2.cat)) Y2.cat<-data.frame(Ycov[,j,drop=FALSE])
-          else Y2.cat<-data.frame(Y2.cat,Ycov[,j,drop=FALSE])
+          if (is.null(Y2.cat)) {
+            Y2.cat<-data.frame(Ycov[,j,drop=FALSE])
+          } else {
+            Y2.cat<-data.frame(Y2.cat,Ycov[,j,drop=FALSE])
+          }
           Y2.numcat<-cbind(Y2.numcat,nlevels(Ycov[,j]))
         }
       }
@@ -109,8 +126,14 @@ jomo.lmer <-
     order.sub<-order.sub[-j.tbd]
     if (!is.null(Y.con)&sum((colnames(Y.con)==clus.name)==1)) Y.con<-data.frame(Y.con[,-which(colnames(Y.con)==clus.name), drop=FALSE])
     if (!is.null(Y2.con)&sum((colnames(Y2.con)==clus.name)==1)) Y2.con<-data.frame(Y2.con[,-which(colnames(Y2.con)==clus.name), drop=FALSE])
-    if (!is.null(Y.cat)&sum((colnames(Y.cat)==clus.name)==1)) Y.cat<-data.frame(Y.cat[,-which(colnames(Y.cat)==clus.name), drop=FALSE])
-    if (!is.null(Y2.cat)&sum((colnames(Y2.cat)==clus.name)==1)) Y2.cat<-data.frame(Y2.cat[,-which(colnames(Y2.cat)==clus.name), drop=FALSE])
+    if (!is.null(Y.cat)&sum((colnames(Y.cat)==clus.name)==1)) {
+      Y.cat<-data.frame(Y.cat[,-which(colnames(Y.cat)==clus.name), drop=FALSE])
+      Y.numcat<-Y.numcat[-which(colnames(Y.cat)==clus.name)]
+    }
+    if (!is.null(Y2.cat)&sum((colnames(Y2.cat)==clus.name)==1)) {
+      Y2.numcat<-Y2.numcat[-which(colnames(Y2.cat)==clus.name)]
+      Y2.cat<-data.frame(Y2.cat[,-which(colnames(Y2.cat)==clus.name), drop=FALSE])
+    }
     if (!is.null(Y.con)&&ncol(Y.con)==0) Y.con <- NULL
     if (!is.null(Y.cat)&&ncol(Y.cat)==0) Y.cat <- NULL
     if (!is.null(Y2.cat)&&ncol(Y2.cat)==0) Y2.cat <- NULL
@@ -150,6 +173,7 @@ jomo.lmer <-
         
       }
     }
+    if (((is.null(Y.con))&&(is.null(Y.cat)&is.null(Y.numcat)))) stop("No level 1 covariates in substantive model. jomo currently supports only models with at least one level 1 variable besides the outcome.\n")
     X=matrix(1,max(nrow(Y.cat),nrow(Y.con)),1)
     if (!is.null(Y2.con)|!is.null(Y2.cat)|!is.null(Y2.aux.con)|!is.null(Y2.aux.cat)) {
       #cat("Level 2 variables must be fully observed for valid inference. \n")
@@ -188,7 +212,8 @@ jomo.lmer <-
     ncolY2con[3]=ncolY2con[1]+max(0,(sum(Y2.numcat)-length(Y2.numcat)))
     ncolYcon[4]=max(0,ncol(Y.cat))
     ncolY2con[4]=max(0,ncol(Y2.cat))
-    stopifnot(((!is.null(Y.con))||(!is.null(Y.cat)&!is.null(Y.numcat)))||((!is.null(Y2.con))||(!is.null(Y2.cat)&!is.null(Y2.numcat))))
+    
+    stopifnot(((!is.null(Y.con))||(!is.null(Y.cat)&!is.null(Y.numcat))))
     if (is.null(u.start)) u.start = matrix(0, nlevels(clus), ncol(Z)*(ncolYcon[1]+max(0,(sum(Y.numcat)-length(Y.numcat)))+max(0,(sum(Y.aux.numcat)-length(Y.aux.numcat))))+(ncolY2con[1]+max(0,(sum(Y2.numcat)-length(Y2.numcat)))+max(0,(sum(Y2.aux.numcat)-length(Y2.aux.numcat)))))
     if (is.null(l2cov.start)) l2cov.start = diag(1, ncol(u.start))
     if (is.null(l2cov.prior)) l2cov.prior = diag(1, ncol(l2cov.start))
@@ -284,41 +309,91 @@ jomo.lmer <-
       colnamycon<-colnames(Y.con)
       Y.con<-data.matrix(Y.con)
       storage.mode(Y.con) <- "numeric"  
+    } else {
+      colnamycon<-NULL
     }
-    if (isnullcat==0) {
-      colnamycat<-colnames(Y.cat)
-      Y.cat<-data.matrix(Y.cat)
-      storage.mode(Y.cat) <- "numeric"  
+    if (isnullcat == 0) {
+      colnamycat <- colnames(Y.cat)
+      Y.cat <- data.matrix(Y.cat)
+      storage.mode(Y.cat) <- "numeric"
+      cnycatcomp<-rep(NA,(sum(Y.numcat)-length(Y.numcat)))
+      count=0
+      for ( j in 1:ncol(Y.cat)) {
+        for (k in 1:(Y.numcat[j]-1)) {
+          cnycatcomp[count+k]<-paste(colnamycat[j],k,sep=".")
+        }
+        count=count+Y.numcat[j]-1
+      }
+    } else {
+      cnycatcomp<-NULL
     }
     if (!is.null(Y2.con)) {
       colnamy2con<-colnames(Y2.con)
       Y2.con<-data.matrix(Y2.con)
       storage.mode(Y2.con) <- "numeric"  
+    } else {
+      colnamy2con<-NULL
     }
     if (isnullcat2==0) {
       colnamy2cat<-colnames(Y2.cat)
       Y2.cat<-data.matrix(Y2.cat)
-      storage.mode(Y2.cat) <- "numeric"  
+      storage.mode(Y2.cat) <- "numeric" 
+      cny2catcomp<-rep(NA,(sum(Y2.numcat)-length(Y2.numcat)))
+      count=0
+      for ( j in 1:ncol(Y2.cat)) {
+        for (k in 1:(Y2.numcat[j]-1)) {
+          cny2catcomp[count+k]<-paste(colnamy2cat[j],k,sep=".")
+        }
+        count=count+Y2.numcat[j]-1
+      }
+    } else {
+      cny2catcomp<-NULL
     }
     if (!is.null(Y.aux.con)) {
       colnamyauxcon<-colnames(Y.aux.con)
       Y.aux.con<-data.matrix(Y.aux.con)
       storage.mode(Y.aux.con) <- "numeric"  
+    }  else {
+      colnamyauxcon<-NULL
     }
-    if (isnullcataux==0) {
-      colnamyauxcat<-colnames(Y.aux.cat)
-      Y.aux.cat<-data.matrix(Y.aux.cat)
-      storage.mode(Y.aux.cat) <- "numeric"  
+    if (isnullcataux == 0) {
+      colnamyauxcat <- colnames(Y.aux.cat)
+      Y.aux.cat <- data.matrix(Y.aux.cat)
+      storage.mode(Y.aux.cat) <- "numeric"
+      cnyauxcatcomp<-rep(NA,(sum(Y.aux.numcat)-length(Y.aux.numcat)))
+      count=0
+      for ( j in 1:ncol(Y.aux.cat)) {
+        for (k in 1:(Y.aux.numcat[j]-1)) {
+          cnyauxcatcomp[count+k]<-paste(colnamyauxcat[j],k,sep=".")
+        }
+        count=count+Y.aux.numcat[j]-1
+      }
+      
+    } else {
+      cnyauxcatcomp<-NULL
     }
     if (!is.null(Y2.aux.con)) {
       colnamy2auxcon<-colnames(Y2.aux.con)
       Y2.aux.con<-data.matrix(Y2.aux.con)
       storage.mode(Y2.aux.con) <- "numeric"  
+    } else {
+      colnamy2auxcon<-NULL
     }
     if (isnullcat2aux==0) {
       colnamy2auxcat<-colnames(Y2.aux.cat)
       Y2.aux.cat<-data.matrix(Y2.aux.cat)
       storage.mode(Y2.aux.cat) <- "numeric"  
+      cny2auxcatcomp<-rep(NA,(sum(Y2.aux.numcat)-length(Y2.aux.numcat)))
+      count=0
+      for ( j in 1:ncol(Y2.aux.cat)) {
+        for (k in 1:(Y2.aux.numcat[j]-1)) {
+          cny2auxcatcomp[count+k]<-paste(colnamy2auxcat[j],k,sep=".")
+        }
+        count=count+Y2.aux.numcat[j]-1
+      }
+      
+    } else {
+      cny2auxcatcomp<-NULL
     }
     Y.cat.tot<-cbind(Y.cat,Y.aux.cat)
     colnamx<-colnames(X)
@@ -362,6 +437,7 @@ jomo.lmer <-
     if (isnullcat2==0) {
       for (i in 1:length(Y2.numcat)) {
         for (j in 1:nrow(Y2)) {
+          cat(j," ", i, "\n")
           if (is.na(Y2.cat[j,i])) {
             Y2i[j,(ncolY2con[1]+h):(ncolY2con[1]+h+Y2.numcat[i]-2)]=NA
           }
@@ -398,7 +474,7 @@ jomo.lmer <-
     if (is.null(a.start)) a.start=50+ncol(Y)
     if (is.null(a.prior)) a.prior=a.start
     
-    if (output!=1) out.iter=nburn+nbetween
+    if (output==0) out.iter=nburn+nbetween
     imp=matrix(0,nrow(Y)*(nimp+1),ncol(Y)+ncY2+4)
     imp[1:nrow(Y),1]=Ysub
     imp[1:nrow(Y),2:(1+ncol(Y))]=Y
@@ -444,15 +520,15 @@ jomo.lmer <-
     
     if (!is.null(Y2)) {
       if (meth=="common") {
-        .Call("jomo2lmerC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, ncolYcon,ncolY2con, out.iter, 0, PACKAGE = "jomo")
+        .Call("jomo2smcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, 1, ncolYcon,ncolY2con, out.iter, 0, 0, PACKAGE = "jomo")
       } else {
-        .Call("jomo2lmerhrC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, ncolYcon,ncolY2con, a.start, a.prior, out.iter, 0, PACKAGE = "jomo")
+        .Call("jomo2hrsmcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, 1,ncolYcon,ncolY2con, a.start, a.prior, out.iter, 0, 0, PACKAGE = "jomo")
       }
     } else {
       if (meth=="common") {
-        .Call("jomo1lmerC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, ncolYcon,out.iter, 0, PACKAGE = "jomo")
+        .Call("jomo1ransmcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, 1, ncolYcon,out.iter, 0, 0, PACKAGE = "jomo")
       } else {
-        .Call("jomo1lmerhrC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, ncolYcon, a.start, a.prior, out.iter, 0, PACKAGE = "jomo")
+        .Call("jomo1ranhrsmcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nburn, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, 1, ncolYcon, a.start, a.prior, out.iter, 0, 0, PACKAGE = "jomo")
       }
     }
     #betapost[,,1]=bpost
@@ -483,7 +559,7 @@ jomo.lmer <-
     if (isnullcat2==0|isnullcat2aux==0) {
       imp[(nrow(Y2)+1):(2*nrow(Y2)),(ncolY2con[1]+ncol(Y)+2):(ncol(Y)+ncY2+1)]=Y2.cat.tot
     }
-    if (output==1) cat("First imputation registered.", "\n")
+    if (output>0) cat("First imputation registered.", "\n")
     for (i in 2:nimp) {
       #Yimp2=matrix(0, nrow(Yimp),ncol(Yimp))
       imp[(i*nrow(clus)+1):((i+1)*nrow(clus)), (ncol(Y)+ncY2+2)]=clus
@@ -491,15 +567,15 @@ jomo.lmer <-
       imp[(i*nrow(X)+1):((i+1)*nrow(X)), (ncol(Y)+ncY2+4)]=i
       if (!is.null(Y2)) {
         if (meth=="common") {
-          .Call("jomo2lmerC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, ncolYcon,ncolY2con, out.iter, 0, PACKAGE = "jomo")
+          .Call("jomo2smcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, 1, ncolYcon,ncolY2con, out.iter, 0, 0, PACKAGE = "jomo")
         } else {
-          .Call("jomo2lmerhrC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, ncolYcon,ncolY2con, a.start, a.prior, out.iter, 0, PACKAGE = "jomo")
+          .Call("jomo2hrsmcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, Y2, Y2imp, Y2imp2, Y2.cat.tot, X, X2, Z, clus,betaY.start,bYpost, betait, beta2it, uit,uY.start,bpost, upost, uYpost, b2post, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, Y2.numcat.tot, 1, ncolYcon,ncolY2con, a.start, a.prior, out.iter, 0, 0, PACKAGE = "jomo")
         }
       } else {
         if (meth=="common") {
-          .Call("jomo1lmerC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, ncolYcon,out.iter, 0, PACKAGE = "jomo")
+          .Call("jomo1ransmcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, 1, ncolYcon,out.iter, 0, 0, PACKAGE = "jomo")
         } else {
-          .Call("jomo1lmerhrC", Ysub, Ysubimp, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, ncolYcon, a.start, a.prior, out.iter, 0, PACKAGE = "jomo")
+          .Call("jomo1ranhrsmcC", Ysub, Ysubimp, 0, submod, order.sub, submod.ran, Y, Yimp, Yimp2, Y.cat.tot, X, Z, clus,betaY.start,bYpost, betait,uit,uY.start,bpost, upost, uYpost, varY.start, vYpost, covit,opost, covuY.start, cuYpost, covuit, cpost, nbetween, varY.prior, covuY.prior, l1cov.prior,l2cov.prior,Y.numcat.tot, 1, ncolYcon, a.start, a.prior, out.iter, 0, 0, PACKAGE = "jomo")
         }
       }
       betapost[,,(i-1)]=bpost
@@ -535,9 +611,28 @@ jomo.lmer <-
       if (isnullcat2==0|isnullcat2aux==0) {
         imp[(i*nrow(X)+1):((i+1)*nrow(X)),(ncolY2con[1]+ncol(Y)+2):(ncol(Y)+ncY2+1)]=Y2.cat.tot
       }
-      if (output==1) cat("Imputation number ", i, "registered", "\n")
+      if (output>0) cat("Imputation number ", i, "registered", "\n")
     }
     
+    cnamycomp<-c(colnamycon, colnamyauxcon, cnycatcomp, cnyauxcatcomp)
+    cnamy2comp<-c(colnamy2con, colnamy2auxcon, cny2catcomp, cny2auxcatcomp)
+    dimnames(betapost)[1] <- list("(Intercept)")
+    dimnames(betapost)[2] <- list(cnamycomp)
+    if (!is.null(Y2)) {
+      dimnames(beta2post)[1] <- list("(Intercept)")
+      dimnames(beta2post)[2] <- list(cnamy2comp)
+    }
+    if (meth=="common") {
+      dimnames(omegapost)[1] <- list(cnamycomp)
+    } else {
+      dimnames(omegapost)[1] <- list(paste(cnamycomp,rep(levels(factor(clus)),each=length(cnamycomp)), sep="."))
+    }
+    dimnames(omegapost)[2] <- list(cnamycomp)
+    colnamcovu<-c(cnamycomp,cnamy2comp)
+    dimnames(covupost)[1] <- list(colnamcovu)
+    dimnames(covupost)[2] <- list(colnamcovu)
+    dimnames(upostall)[1]<-list(levels(factor(clus)))
+    dimnames(upostall)[2]<-list(colnamcovu)
     betaYpostmean<-apply(betaYpost, c(1,2), mean)
     varYpostmean<-mean(varYpost)
     covuYpostmean<-apply(covuYpost, c(1,2), mean)
@@ -547,7 +642,12 @@ jomo.lmer <-
     upostmean<-apply(upostall, c(1,2), mean)
     omegapostmean<-apply(omegapost, c(1,2), mean)
     covupostmean<-apply(covupost, c(1,2), mean)
-    if (output==1) {
+    colnames(betaYpostmean)<-rownames(summary(fit.cr)$coefficients)
+    rownames(betaYpostmean)<-colnamysub
+    colnames(covuYpostmean)<-rownames(covuYpostmean)<-colnames(uYpostmean)<-dimnames(summary(fit.cr)$varcor[[1]])[[1]]
+    rownames(uYpostmean)<-levels(factor(clus))
+    
+    if (output>0) {
       cat("The posterior mean of the substantive model fixed effects estimates is:\n")
       print(betaYpostmean)
       cat("The posterior mean of the substantive model residual variance is:\n")
@@ -556,18 +656,20 @@ jomo.lmer <-
       print(covuYpostmean)
       cat("The posterior mean of the substantive model random effects estimates is:\n")
       print(uYpostmean)
-      cat("The posterior mean of the fixed effects estimates is:\n")
-      print(betapostmean)
-      if (!is.null(Y2)) {
-        cat("The posterior mean of the level 2 fixed effects estimates is:\n")
-        print(beta2postmean)
+      if (output==2) {
+        cat("The posterior mean of the fixed effects estimates is:\n")
+        print(betapostmean)
+        if (!is.null(Y2)) {
+          cat("The posterior mean of the level 2 fixed effects estimates is:\n")
+          print(beta2postmean)
+        }
+        cat("The posterior mean of the random effects estimates is:\n")
+        print(upostmean)
+        cat("The posterior mean of the level 1 covariance matrix is:\n")
+        print(omegapostmean)
+        cat("The posterior mean of the level 2 covariance matrix is:\n")
+        print(covupostmean)
       }
-      cat("The posterior mean of the random effects estimates is:\n")
-      print(upostmean)
-      cat("The posterior mean of the level 1 covariance matrix is:\n")
-      print(omegapostmean)
-      cat("The posterior mean of the level 2 covariance matrix is:\n")
-      print(covupostmean)
     }
     imp<-data.frame(imp)
     if (isnullcat==0) {
